<?php
/**
 * @author Tiarg Team
 * @copyright Copyright (c) 2021 Tiarg
 * @package Tiargsa_CorreoArgentino
 */

namespace Tiargsa\CorreoArgentino\Service;

use Laminas\Http\Request as RequestAlias;
use Tiargsa\CorreoArgentino\Helper\Data as correoHelper;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\DataObject;
use Magento\Framework\Webapi\Rest\Request;
use Psr\Log\LoggerInterface;

class CorreoApiService
{
    /**
     * @var correoHelper
     */
    private $helper;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    public function __construct(
        correoHelper $data,
        LoggerInterface $logger
    ) {
        $this->helper = $data;
        $this->logger = $logger;
    }

    public function authorization()
    {
        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        $response = $this->doRequest($this->helper->getLoginUrl(), [
            'header' => [
                'authorization: Apikey ' . $apikey,
                'agreement:' . $agreement
            ]
        ]);

        if($response->getStatusCode() === 404) {
            return json_encode([
                'message' => "404 Not Found"
            ]);
        }
        return $response->getReason();

    }

    /**
     * @param $tracking
     * @return DataObject
     */
    public function getShippingHistory($tracking)
    {
        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        $shippingByNumber =  $this->helper->getShippingByNumberUrl() . $tracking;
        return $this->getDataFromResponse($this->doRequest(
            $shippingByNumber,
            [
                'header' => [
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
                ]
            ]
        ));
    }

    /**
     * @return DataObject
     */
    public function getCancel($tracking)
    {
        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        $cancelUrl = $this->helper->getCancelUrl(). $tracking . "/cancel";
        return $this->getDataFromResponse($this->doRequest(
            $cancelUrl,
            [
                'header' => [
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
                ]
            ],
            RequestAlias::METHOD_PATCH
        ));
    }

    /**
     * @return DataObject
     */
    public function getLocations()
    {
        $locationUrl = $this->helper->getLocationUrl() . 'pickup_availability=true';

        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        return $this->getDataFromResponse($this->doRequest(
            $locationUrl,
            [
                'header' => [
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
                ]
            ]
        ));
    }

    /**
     * @param DataObject $data
     * @return DataObject
     */
    public function getRates(DataObject $data)
    {

        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        $response = $this->getDataFromResponse($this->doRequest(
            $this->helper->getRatesUrl(),
            [
                'body' => $data->getData(),
                'header' => [
                    'Content-Type: application/json',
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
                ]
            ],
            Request::HTTP_METHOD_POST
        ));

        if (!$this->helper->isCotizadorOn())
        {
            if (isset($response['rates'])) {
                foreach ($response['rates'] as &$rate) {
                    $rate['totalPrice'] = "0";
                }
            }
        }

        return $response;
    }

    /**
     * @param DataObject $data
     * @return DataObject
     */
    public function createOrder(DataObject $data)
    {
        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        return $this->getDataFromResponse($this->doRequest(
            $this->helper->getCreateOrderUrl(),
            [
                'body' => $data->getData(),
                'header' => [
                    'Content-Type: application/json',
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
            ]
        ], Request::HTTP_METHOD_POST));
    }

    /**
     * @param string $tracking
     * @return DataObject
     */
    public function getLabel(string $tracking)
    {
        $labelUrl = $this->helper->getLabelUrl();

        $agreement = $this->helper->getAgreement();
        $apikey = $this->helper->getApiKey();

        $params = [
            [
                "sellerId" => '',
                "trackingNumber" => $tracking
            ]
        ];
        return $this->getDataFromResponse($this->doRequest(
            $labelUrl,
            [
                'body' => $params,
                'header' => [
                    'Content-Type: application/json',
                    'authorization: Apikey ' . $apikey,
                    'agreement:' . $agreement
                ]
            ],
            Request::HTTP_METHOD_POST
        ));
    }

    /**
     * @param DataObject $response
     * @return DataObject
     */
    private function getDataFromResponse(DataObject $response)
    {
        if ($response->getStatusCode() == 200) {
            return $response->getValue();
        }
        return $response->getReason();
    }

    /**
     * @param $uri
     * @param array $params
     * @param string $requestMethod
     * @param bool $parseToArray
     * @return DataObject
     */
    private function doRequest(
        $uri,
        $params = [],
        $requestMethod = Request::HTTP_METHOD_GET,
        $parseToArray = true
    ) {
        $response = new DataObject();
        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $uri,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => $requestMethod,
        ]);
        if (isset($params['header'])) {
            curl_setopt($curl, CURLOPT_HTTPHEADER, $params['header']);
        }
        if (isset($params['body'])) {
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($params['body']));
        }

        $lala = curl_getinfo($curl, CURLOPT_HTTPHEADER);
        $curlResponse = curl_exec($curl);

        $statusCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        curl_close($curl);
        if ($statusCode < 200 || $statusCode >= 300) {
            $response->setStatusCode($statusCode);
            $response->setReason(strval($curlResponse));
        } else {
            $response->setStatusCode(200);
            $response->setValue(json_decode($curlResponse, true));
            $response->setHeaders($params['header']);
        }
        return $response;
    }
}
